/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package socialnetwork.entities;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;

/**
 * Represents user in our network (student or lecturer).
 * @author gsnewmark
 */
@Entity
public class NetworkUser implements Serializable, IUser {
    private static final long serialVersionUID = 1L;
    private Long id;
    private String name;
    private String description;
    private String login;
    private String password;
    private boolean isAdmin;
    private Set<SubjectGroup> subjectGroups;
    private Set<Blog> blogs;
    private Set<Topic> topics;
    private Set<Post> posts;

    public NetworkUser() {
        subjectGroups = new HashSet<SubjectGroup>();
        blogs = new HashSet<Blog>();
        topics = new HashSet<Topic>();
        posts = new HashSet<Post>();
        this.isAdmin = false;
    }
    
    @Override
    @Id @GeneratedValue(strategy = GenerationType.AUTO)
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }
    
    @Override
    @Column(nullable=false)
    public String getDescription() {
        return description;
    }

    public void setDescription(String _description) {
        this.description = _description;
    }

    @Column (nullable=false)
    public boolean isIsAdmin() {
        return isAdmin;
    }

    public void setIsAdmin(boolean isAdmin) {
        this.isAdmin = isAdmin;
    }
    
    @Override
    @Column
    public String getLogin() {
        return login;
    }

    public void setLogin(String _login) {
        this.login = _login;
    }

    @Override
    @Column(nullable=false)
    public String getName() {
        return name;
    }

    public void setName(String _name) {
        this.name = _name;
    }

    @Override
    @Column
    public String getPassword() {
        return password;
    }

    public void setPassword(String _password) {
        this.password = _password;
    }

    @Override
    @ManyToMany(mappedBy="userList") 
    public Set<SubjectGroup> getSubjectGroups() {
        return subjectGroups;
    }

    public void setSubjectGroups(Set<SubjectGroup> _subjectGroups) {
        this.subjectGroups = _subjectGroups;
    }
    
    @OneToMany(mappedBy="creator")
    public Set<Blog> getBlogs() {
        return blogs;
    }

    public void setBlogs(Set<Blog> blogs) {
        this.blogs = blogs;
    }

    @OneToMany(mappedBy="creator")
    public Set<Topic> getTopics() {
        return topics;
    }

    public void setTopics(Set<Topic> topics) {
        this.topics = topics;
    }

    @OneToMany(mappedBy="creator")
    public Set<Post> getPosts() {
        return posts;
    }

    public void setPosts(Set<Post> posts) {
        this.posts = posts;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final NetworkUser other = (NetworkUser) obj;
        if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }
    
    @Override
    public String toString() {
        return "NetworkUser{" + "id=" + id + ", name=" + name + ", description=" + description + ", login=" + login + ", password=" + password + ", isAdmin=" + isAdmin + '}';
    }
    
}
