/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package socialnetwork.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.OneToOne;

/**
 * Represents subject group in our social network.
 * @author gsnewmark
 */
@Entity
public class SubjectGroup implements IGroup, Serializable {
    
    private Long id;
    private String name;
    private Set<IUser> userList;
    private Blog blog;
    
    public SubjectGroup() {
        userList = new HashSet<IUser>();
    }
    
    @Override
    @Id @GeneratedValue
    public Long getId() {
        return id;
    }

    public void setId(Long _id) {
        this.id = _id;
    }

    @Override
    @Column(nullable=false)
    public String getName() {
        return name;
    }

    public void setName(String _name) {
        this.name = _name;
    }
    
    @Override
    @ManyToMany(targetEntity=NetworkUser.class)
    public Set<IUser> getUserList() {
        return this.userList;
    };
    
    /**
     * @return list of users with admin rights (i. e., lecturers)
     */
    public List<IUser> getAdminList() {
        List result = new ArrayList();
        for (IUser u : getUserList()) {
            if (((NetworkUser)u).isIsAdmin()) {
                result.add(u);
            }
        }
        return result;
    } 
    
    /**
     * @return list of users without admin rights (i. e., students)
     */
    public List<IUser> getStudentList() {
        List result = new ArrayList();
        for (IUser u : getUserList()) {
            if (!((NetworkUser)u).isIsAdmin()) {
                result.add(u);
            }
        }
        return result;
    } 

    public void setUserList(Set<IUser> _userList) {
        this.userList = _userList;
    }
    
    @OneToOne
    public Blog getBlog() {
        return blog;
    }

    public void setBlog(Blog _blog) {
        this.blog = _blog;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final SubjectGroup other = (SubjectGroup) obj;
        if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 29 * hash + (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return "SubjectGroup{" + "id=" + id + ", name=" + name + '}';
    }
    
}
