/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package socialnetwork.managedbeans;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.faces.context.FacesContext;
import socialnetwork.ejb.SocNetworkDAO;
import socialnetwork.entities.IUser;
import socialnetwork.entities.Post;
import socialnetwork.entities.SubjectGroup;
import socialnetwork.entities.Topic;

/**
 * Governs Group page.
 * @author gsnewmark
 */
public class GroupManagedBean {
    @EJB
    private SocNetworkDAO socNetworkDAO;
    private SubjectGroup _group;
    private IUser _student;
    private String _message;
    private String _topicName;
    private long _groupId;

    /** Creates a new instance of GroupManagedBean */
    public GroupManagedBean() {
        this._student = ((LoginBackingBean) FacesContext.getCurrentInstance().
                getExternalContext().getSessionMap().get("loginBackingBean")).
                getStudent();
        _groupId = ((Long) FacesContext.getCurrentInstance().
                getExternalContext().getSessionMap().get("gid"));
    }
    
    @PostConstruct
    void init() {
        this._group = socNetworkDAO.getGroupFromDB(_groupId);
    }

    public SubjectGroup getGroup() {
        return _group;
    }

    public void setGroup(SubjectGroup _group) {
        this._group = _group;
    }

    public String getMessage() {
        return _message;
    }

    public void setMessage(String _message) {
        this._message = _message;
    }

    public IUser getStudent() {
        return _student;
    }

    public void setStudent(IUser _student) {
        this._student = _student;
    }

    public String getTopicName() {
        return _topicName;
    }

    public void setTopicName(String _topicName) {
        this._topicName = _topicName;
    }
    
    /**
     * Checks whether the entered user belongs to group from this page.
     * @return true if user is member of group
     */
    public boolean isMember(IUser student) {
        return (student == null || student.equals(this._student)) && 
                this._group.getUserList().contains(this._student);
    }
    
    /**
     * Adds new message from page to DB.
     * @param topic where to add message
     * @return true if message was added
     */
    public boolean onMessageAdd(Topic topic) {
        Post newPost = new Post();
        newPost.setPostContents(getMessage());
        socNetworkDAO.addPostToDB(newPost);
        socNetworkDAO.addAuthorToPost(newPost, getStudent());
        socNetworkDAO.addTopicToPost(newPost, topic);
        setMessage(null);
        return true;
    }
    
    /**
     * Adds new message from page to DB.
     * @param topic where to add message
     * @return true if message was added
     */
    public boolean onTopicAdd() {
        Post newPost = new Post();
        newPost.setPostContents(getMessage());
        socNetworkDAO.addPostToDB(newPost);
        Topic newTopic = new Topic();
        newTopic.setTopicName(getTopicName());
        socNetworkDAO.addTopicToDB(newTopic);
        socNetworkDAO.addBlogHolderToTopic(newTopic, this._group.getBlog());
        socNetworkDAO.addAuthorToTopic(newTopic, getStudent());
        socNetworkDAO.addAuthorToPost(newPost, getStudent());
        socNetworkDAO.addTopicToPost(newPost, newTopic);
        setMessage(null);
        setTopicName(null);
        return true;
    }
    
    /**
     * Removes user from group.
     * @param student student to remove
     * @return true if user was deleted
     */
    public boolean removeFromGroup(IUser student) {
        socNetworkDAO.removeUserFromSubjectGroup(student, getGroup());
        return true;
    }
    
    /**
     * Moves user back to list of groups.
     * @return where to go
     */
    public String goBack() {
        return "groupsList";
    }
    
    /**
     * Adds user to this group.
     * @param student student to add
     * @return true if user was added
     */
    public boolean addToGroup() {
        socNetworkDAO.addUserToSubjectGroup(getStudent(), getGroup());
        return true;
    }
}
