/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package socialnetwork.servlets;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.ejb.EJB;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import socialnetwork.ejb.SocNetworkDAO;

/**
 *
 * @author gsnewmark
 */
public class PageController extends HttpServlet {
    @EJB
    private SocNetworkDAO socNetworkDAO;
    
    private static String URI_HEADER;
    // key - name of init parameter <...>_URI_PART, value - array of 2 elements:
    // value of this parameter and value of parameter <same...>_PAGE
    private static Map<String, TwoStringHolder> PAGES = new HashMap<String, TwoStringHolder>();
    
    /**
     * Utility class for building PAGES.
     */
    private static class TwoStringHolder {
        private String first;
        private String second;

        public TwoStringHolder(String first, String second) {
            this.first = first;
            this.second = second;
        }
        
        public String getFirst() {
            return first;
        }

        public void setFirst(String first) {
            this.first = first;
        }

        public String getSecond() {
            return second;
        }

        public void setSecond(String second) {
            this.second = second;
        }

        @Override
        public String toString() {
            return "TwoStringHolder{" + "first=" + first + ", second=" + second + '}';
        }
    }

    @Override
    public void init() throws ServletException {
        super.init();
        // save reference to ejb
        getServletContext().setAttribute("DAO", socNetworkDAO);
        fillPages(getServletConfig());
        URI_HEADER = getServletConfig().getInitParameter("URI_HEADER");
        //socNetworkDAO.createContents();
    }

    @Override
    public void destroy() {
        super.destroy();
    }
    
    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        PrintWriter out = response.getWriter();
        try {
            String uri = request.getRequestURI();
            String pagePart = findPage(uri);
            if (pagePart == null || (
                    !pagePart.equalsIgnoreCase(PAGES.get("LOGIN_LOGIC_URI_PART").getFirst()) &&
                    !pagePart.equalsIgnoreCase(PAGES.get("STUDENT_LOGIN_URI_PART").getFirst()) &&
                    !pagePart.equalsIgnoreCase(PAGES.get("LOGIN_URI_PART").getFirst())) &&
                (request.getSession().getAttribute("loggedIn") == null ||
                    !(Boolean)request.getSession().getAttribute("loggedIn"))) {
                openPage(PAGES.get("INDEX_URI_PART").getFirst(), request, response);
            } else {
                openPage(pagePart, request, response);
            }
        }
        finally {            
            out.close();
        }
    }
    
    /**
     * Finds page part in link.
     * @param uri opened link 
     * @return page part of this link
     */
    private static String findPage(String uri) {
        String uriPatternString = "^" + URI_HEADER + "(/.*)" + "$";
        Pattern uriPattern = Pattern.compile(uriPatternString);

        Matcher uriMatcher = uriPattern.matcher(uri);
        String pagePart = null;
        while (uriMatcher.find()) {
            pagePart = uriMatcher.group(1);
        }

        return pagePart;
    }
    
    /**
     * Redirects to correct page or prints default information.
     * @param pagePart address of page
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    private void openPage(String pagePart, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        PrintWriter out = response.getWriter();

        String pageToLoad = findPageToLoad(pagePart);
        
        if (pageToLoad != null) {
            RequestDispatcher requestDispatcher = getServletContext().
                    getRequestDispatcher(pageToLoad);
            requestDispatcher.forward(request, response);
        } else {
            out.println("<html>");
            out.println("<head>");
            out.println("<title>Error</title>");  
            out.println("</head>");
            out.println("<body>");
            out.println("<h1>ERROR 404: Page not found</h1>");
            out.println("</body>");
            out.println("</html>");
        }
    }
    
    /**
     * Finds what page needs to be loaded from PAGES map.
     * @param pagePart invoked link
     * @return address of page to load
     */
    private static String findPageToLoad(String pagePart) {
        String pageToLoad = null;
        
        if (pagePart != null) {
            for (String uri:PAGES.keySet()) {
                if (pagePart.equalsIgnoreCase(PAGES.get(uri).getFirst())) {
                    pageToLoad = PAGES.get(uri).getSecond();
                    break;
                }
            }
        }
        
        return pageToLoad;
    }
    
    /**
     * Fills map of all pages this servlet governs.
     * In servlets initialization parameters exist pairs of <name>_URI_PART
     * <name>_PAGE for each page <name> that servlet controls.
     * @param sc servlet's initial config
     */
    private static void fillPages(ServletConfig sc) {
        String pageUriPatternString = "^(\\w+)_URI_PART$";
        Pattern pageUriPattern = Pattern.compile(pageUriPatternString);
        
        
        for (String name:Collections.list(sc.getInitParameterNames())) {
            Matcher pageUriMatcher = pageUriPattern.matcher(name);
            String pageUri = null;
            while (pageUriMatcher.find()) {
                pageUri = pageUriMatcher.group(1);
            }
            if (pageUri != null) {
                PAGES.put(name, new TwoStringHolder(
                        sc.getInitParameter(name), 
                        sc.getInitParameter(pageUri + "_PAGE")));
            }
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>
}
